#!/usr/bin/env python3

import numpy as np
from hiopy._distribute_work import distribute_work

try:
    from pprint import pp
except ImportError:
    print("Cannot find pprint falling back to print")
    pp = print


class DummyDataVar:
    def __init__(self, name, shape, chunksize=None):
        self.name = name
        self.shape = shape
        self.chunks = (1, chunksize or shape[1])
        self.attrs = {"_ARRAY_DIMENSIONS": ["time", "cell"]}
        self.dtype = np.dtype("float64")

    def __repr__(self):
        return self.name


def test_sequential():
    print("Test sequential case")
    var1 = DummyDataVar("var1", [10, 10])
    var2 = DummyDataVar("var2", [10, 10])
    result = distribute_work({"g1": [var1], "g2": [var2]}, 1)
    pp(result)
    assert len(result) == 1
    assert result[0] == [("g1", [var1], slice(None)), ("g2", [var2], slice(None))]


def test_dist_groups():
    print("Test distributing groups")
    var1 = DummyDataVar("var1", [10, 10])
    var2 = DummyDataVar("var2", [10, 10])
    result = distribute_work({"g1": [var1], "g2": [var2]}, 2)
    pp(result)
    assert len(result) == 2
    assert result[0] == [("g1", [var1], slice(None))]
    assert result[1] == [("g2", [var2], slice(None))]


def test_dist_chunks():
    print("Test distributing chunks")
    var = DummyDataVar("var", [10, 10], 5)
    result = distribute_work({"g": [var]}, 2)
    pp(result)
    assert len(result) == 2
    assert result[0] == [("g", [var], slice(0, 5))]
    assert result[1] == [("g", [var], slice(5, 10))]


if __name__ == "__main__":
    test_sequential()
    test_dist_groups()
    test_dist_chunks()
