#!/bin/bash
set -o errexit -o pipefail -o noclobber -o nounset

DEPENDENCY_TO_BUILD="all"
while [ $# -gt 0 ]; do
    case "$1" in
        --coyote-root)
            coyote_ROOT=$2
            ABSOLUTE_coyote_ROOT=$(readlink -f "${coyote_ROOT}")
            shift 2
            ;;
        --dependency)
            DEPENDENCY_TO_BUILD=$2
            shift 2
            ;;
        *)
            echo "Error: unknown argument '$1'. Call this script with --coyote-root and path to your coyote-root"
            exit 1
            ;;
    esac
done

echo "Absolute path provided as the coyote root: $ABSOLUTE_coyote_ROOT"

# take the right compiler
COMPILER="cce-17.0.1"
module purge
module load PrgEnv-cray cce/17.0.1 craype-x86-milan cray-mpich/8.1.29 cray-hdf5 cray-netcdf cray-libsci libfabric
unset INCLUDE_PATH_X86_64

export CC="cc"
export CXX="CC"
export FC="ftn"

BUILD_PATH=$(pwd)/$COMPILER
INSTALL_PATH=$BUILD_PATH/install
mkdir -p "$BUILD_PATH"
pushd "$BUILD_PATH"


# recommended to use a compute node for the build process with > 8 threads
THREADS=64

# setup a py-env for further tests and development
module load cray-python/3.11.7


# build dependencies for coyote
echo "=== Building coyote dependencies ==="
$ABSOLUTE_coyote_ROOT/scripts/setup_devenv/build_dependencies.sh -j $THREADS --dependency $DEPENDENCY_TO_BUILD --prefix_path $INSTALL_PATH


VENV_PATH=$BUILD_PATH/py_venv
python -m venv $VENV_PATH
source $VENV_PATH/bin/activate
pip install -r ${ABSOLUTE_coyote_ROOT}/requirements-dev.txt

# build coyote
echo "=== Building coyote ==="
CC="${CC}" CXX="${CXX}" FC="${FC}" cmake $ABSOLUTE_coyote_ROOT -DCMAKE_PREFIX_PATH=$INSTALL_PATH -DCMAKE_BUILD_TYPE=Debug
cmake --build . -j $THREADS

cp $BUILD_PATH/python/coyote.cpython-311-x86_64-linux-gnu.so $VENV_PATH/lib/python3.11/site-packages/
export PYTHONPATH=${BUILD_PATH}/python:${ABSOLUTE_coyote_ROOT}/apps
echo $PYTHONPATH

if python -c "import coyote; from hiopy._data_handler import DataHandler" &> /dev/null ; then
    echo "Coyote libraries installed and usable successfully"
else
    echo "Something went wrong in appropriate installation of coyote and hiopy"
fi

popd
