#include <chrono>
#include <exception>
#include <iostream>

#include "../src/tsqueue.hpp"

using namespace std::chrono_literals;

void test_priority() {
    TSQueue<int, int> q;

    q.enqueue(3, 3);
    q.enqueue(5, 5);
    q.enqueue(1, 1);
    q.enqueue(-100, -100);
    q.enqueue(100, 100);
    q.end();

    int last  = 9999;
    int count = 0;
    while (true) {
        try {
            int i = q.dequeue();
            count++;
            std::cout << i << std::endl;
            if (i > last)
                throw std::runtime_error("queue provided elements in wrong order.");
            last = i;
        } catch (QueueTerminated) {
            break;
        }
    }
    if (count != 5)
        throw std::runtime_error("not all element provided");
}

void test_thread_safety() {
    TSQueue<int, int> q;

    std::thread t([&]() {
        for (int i = 0; i < 10; ++i) {
            q.enqueue(i * i, 0);
            std::this_thread::sleep_for(5ms);
        }
        q.end();
    });

    for (int i = 0; i < 10; ++i) {
        std::cerr << q.dequeue() << std::endl;
    }

    t.join();

    try {
        q.dequeue();
    } catch (QueueTerminated) {
        return;
    }
    throw std::runtime_error("Temination failed.");
}

int main() {
    std::cerr << "Testing thread_safety" << std::endl;
    test_thread_safety();
    std::cerr << "Testing priorities" << std::endl;
    test_priority();
}
